/*  --------------------------------------------------- !
! The current code is to:                               !
! (1) do matrix-matrix products.                        !
! (2) see the behavior of the threaded MKL routine .    !
! (3) measure attainable performance in MFLOP/s.        !
! (4) understand comput-bound application.              !
! (5) Precision: real DP.                               !
!-------------------------------------------------------!
! Xiaoxu Guan at LSU HPC.                               !
! April 20, 2017                                        !
!----------------------------------------------------- */

#include <omp.h>
#include <mkl.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

// extern double mysecond();

int main() {
	
const int nsize = 2000;
const int nsize2 = nsize*nsize;
const int iteration = 20;
const double invs_nsize = 1.0/(double)(nsize);
const double alpha = 1.0, beta = 0.0;
double *matrix_a, *matrix_b, *matrix_c;
char trans_a = 'N', trans_b = 'N';
int i,j,k;
double tmp,time_start,time_end,elapsed_time,perf,dsize;

// allocate the matrices.
    matrix_a = (double *) malloc( nsize2*sizeof(double) );     
    matrix_b = (double *) malloc( nsize2*sizeof(double) );     
    matrix_c = (double *) malloc( nsize2*sizeof(double) );     
    if(matrix_a == NULL || matrix_b == NULL || matrix_c == NULL) printf("Failure to allocate array matrix!\n");

// initialize the matrices.
    for (i=0; i<nsize2; i++)
	{ matrix_a[i] = sin((double)(i))*invs_nsize; 
          matrix_b[i] = cos((double)(i))*invs_nsize; 
          matrix_c[i] = 0.0; 
        }


// start a timer.
   time_start = omp_get_wtime();

// repeat "iteration" times: C = A*B.
    for (k=0; k<iteration; k++)
      {  
//        printf("Number of the threads = %d\n", omp_get_num_threads());
        cblas_dgemm(CblasRowMajor, CblasNoTrans, CblasNoTrans, nsize, nsize, nsize, \
                    alpha, matrix_a, nsize, matrix_b, nsize, beta, matrix_c, nsize);
      }

   time_end = omp_get_wtime();
   elapsed_time = time_end - time_start;
   printf("Elapsed time (sec) is %.5f\n", elapsed_time);

   dsize = (double)nsize;
   perf= 2.0 * dsize * dsize * dsize * (double) (iteration) / elapsed_time / 1.e+6; 
   printf("matrix size = %d\n",nsize);
   printf("performance (MFLOP/s) = %.5f\n", perf); 

// print the a corner of the C matrix elements.
    for (i=0; i<=5; i++)
         printf("%.5e\n", matrix_c[i]); 

    printf("\n");

    free(matrix_a);
    free(matrix_b);
    free(matrix_c);

}


