/*  --------------------------------------------------- !
! The current code is to:                               !
! (1) do the matrix transposition.                      !
! (2) see the behavior of the threaded MKL routine .    !
! (3) measure the throughput in GB/s.                   !
! (4) understand memory-bound application.              !
! (5) Precision: real DP.                               !
!-------------------------------------------------------!
! Xiaoxu Guan at LSU HPC.                               !
! April 21, 2017                                        !
!----------------------------------------------------- */

#include <omp.h>
#include <mkl.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

int main() {
	
const int nsize = 4000;
const int nsize2 = nsize*nsize;
const int iteration = 1000;
const double invs_nsize = 1.0/(double)(nsize);
const double alpha = 1.0;
double *matrix_a, *matrix_b;
char trans_a = 'N';
int i,j,k;
double tmp,time_start,time_end,elapsed_time,perf,dsize;

// allocate the matrices.
    matrix_a = (double *) malloc( nsize2*sizeof(double) );     
    matrix_b = (double *) malloc( nsize2*sizeof(double) );     
    if(matrix_a == NULL || matrix_b == NULL ) printf("Failure to allocate array matrix!\n");

// initialize the matrices.
    for (i=0; i<nsize2; i++)
	{ matrix_a[i] = sin((double)(i))*invs_nsize; 
          matrix_b[i] = 0.0; 
        }


// start a timer.
   time_start = omp_get_wtime();

// repeat iteration times: B = A^T.
    for (k=0; k<iteration; k++)
      {  
        mkl_domatcopy('R', 'T', nsize, nsize, alpha, matrix_a, nsize, matrix_b, nsize);
      }

   time_end = omp_get_wtime();
   elapsed_time = time_end - time_start;
   printf("Elapsed time (sec) is %.5f\n", elapsed_time);

   dsize = (double)nsize;
   perf= 2.0 * dsize * dsize * 8.0 * (double) (iteration) / elapsed_time / 1.e+9; 
   printf("matrix size = %d\n",nsize);
   printf("performance (GB/s) = %.5f\n", perf); 


// print a corner in the 1st row of the C matrix.
    for (i=0; i<=5; i++)
         printf("%.5e\n", matrix_b[i]); 

    printf("\n");

    free(matrix_a);
    free(matrix_b);

}

