/* ================================================!
! Calculate \pi.                                   !
!--------------------------------------------------!
! (1) Pure MPI version.                            !
! (2) MPI_REDUCE().                                !
!--------------------------------------------------!
!  Xiaoxu Guan at LSU HPC                          !
!  May 12, 2017                                    !
!================================================ */

#include <mpi.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdbool.h>
#define master_id 0
#define idread 5, idwrite 6, master_id 0
#define MAX(a,b) ((a) > (b) ? a : b)
#define MIN(a,b) ((a) < (b) ? a : b)

int   my_id,numprocs,my_len;
int   version,subversion;
char  machine_name[MPI_MAX_PROCESSOR_NAME];


int main (int argc, char *argv[])

{ double temp,tmp,xi,fsum,pi,h;
  double time_s,time_e;
  int    itemp,chunk_floor,chunk_ceili,rmn;
  int    istart,iend,chunk,i;
  int    nsize;


  MPI_Init(&argc, &argv);
  MPI_Comm_size(MPI_COMM_WORLD,&numprocs);
  MPI_Comm_rank(MPI_COMM_WORLD,&my_id);
  
  MPI_Get_processor_name(machine_name,&my_len);
  MPI_Get_version(&version,&subversion);

  if(!my_id) { datetime("\nJob submitted on"); }

  MPI_Barrier(MPI_COMM_WORLD);

  if(argc != 2) {
     if (!my_id) { printf("Number of input arguments is %d\n", argc-1);
                   printf("It must be 1.\nQuit!"); }
     MPI_Finalize();
     exit(1);
  } 
  else {
     nsize = atoi(argv[1]);
     if(!my_id) { printf("Number of points: %d\n", nsize); }
  }

  MPI_Barrier(MPI_COMM_WORLD);

// domain decomposition.
      temp = (double)(nsize) / (double)(numprocs);
      chunk_floor = (int) floor(temp);
      chunk_ceili = (int) ceil(temp);

      rmn =  nsize % numprocs;

// determine the start and end indices: istar and iend.
      if( rmn > 0 && my_id <= rmn-1 ) 
         {
          chunk  = chunk_ceili;
          istart = my_id * chunk + 1;
          iend   = istart + chunk - 1; 
         }
      else if( rmn > 0 && my_id > rmn-1 ) 
         {
          chunk  = chunk_floor;
          itemp  = chunk_ceili * rmn;
          istart = itemp + 1 + (my_id - rmn ) * chunk;
          iend   = istart + chunk - 1;
         }
      else if(rmn == 0) 
         {
          chunk = chunk_floor;
          istart = my_id * chunk + 1;
          iend   = istart + chunk - 1;
         }

   printf("%d %d %d %d %d\n", my_id, istart, iend, chunk, nsize);    
   MPI_Barrier(MPI_COMM_WORLD);

   if(!my_id) { time_s = MPI_Wtime(); }

// sum over points.
       h = 1.0 / (double)(nsize);
       fsum = 0.0;
   for(i=istart; i<=iend; i++) { 
       xi = h * ((double)(i)-0.5);
       tmp = 1.0 + xi * xi;
       fsum = fsum + 1.0 / tmp;
     }

   fsum = 4.0 * h * fsum;
   pi = 0.0;
   MPI_Reduce(&fsum,&pi,1,MPI_DOUBLE_PRECISION,MPI_SUM,0,MPI_COMM_WORLD);

   if(!my_id) { 
      time_e = MPI_Wtime(); 
      printf("\nElapsed time (sec) = %10.5f\n", time_e-time_s);
      printf("pi = %18.15f\n", pi); 
      datetime("\nJob finished on"); 
   }

   MPI_Finalize();
   return 0;

}


//================================//
//    dat and time function       //
//================================//

   datetime(char message[])
{

   time_t t;
   t = time(NULL);
   printf("%s %s\n",message, ctime(&t));

}
