!=======================================================!
! The current code is to:                               !
! (1) do matrix-matrix products.                        !
! (2) see the behavior of the threaded MKL routine .    !
! (3) measure attainable performance in MFLOP/s.        !
! (4) understand comput-bound application.              !
! (5) Precision: real DP.                               !
!-------------------------------------------------------!
! Xiaoxu Guan at LSU HPC.                               !
! May 12, 2017                                          !
!=======================================================!

      program mkl_dgemm
      implicit none

      integer, parameter :: nsize = 2000
      integer, parameter :: nsize2 = nsize*nsize
      integer, parameter :: iteration = 20
      real(kind=8), parameter :: invs_nsize = 1.d0/dble(nsize)
      real(kind=8), parameter :: alpha = 1.d0, beta = 0.d0
      real(kind=8), dimension(:), allocatable ::  matrix_a, matrix_b, matrix_c
      character(len=1), parameter :: trans_a = 'N', trans_b = 'N'
      integer :: i,j,k,ii
      real(kind=8)  :: time_start,time_end,elapsed_time,perf,dsize
      real(kind=8)  :: omp_get_wtime
      integer       :: omp_get_num_threads

! allocate the matrices, same as c code.
      ALLOCATE( matrix_a(0:nsize2-1) )    
      ALLOCATE( matrix_b(0:nsize2-1) )    
      ALLOCATE( matrix_c(0:nsize2-1) )    

! initialize the matrices.
         k = -1
      do i = 0, nsize-1  ! column
      do j = 0, nsize-1  ! row
         k = k + 1
         ii = j * nsize + i
         matrix_a(k) = sin(dble(ii))*invs_nsize
         matrix_b(k) = cos(dble(ii))*invs_nsize
         matrix_c(k) = 0.d0 
      end do 
      end do 

! start a timer.
      time_start = omp_get_wtime()

!! repeat "iteration" times: C = A*B.
!! #pragma omp parallel for num_threads(4)
!! #pragma omp parallel for 
      do k = 0, iteration-1
!!         write(*,'(1x,a,i3)') "Number of the threads = ", omp_get_num_threads()
         call DGEMM(trans_a,trans_b,nsize,nsize,nsize,alpha,matrix_a,nsize,      &
     &        matrix_b,nsize,beta,matrix_c,nsize)
      end do

      time_end = omp_get_wtime()
      elapsed_time = time_end - time_start
      write(*,'(1x,a,f12.5)') "Elapsed time (sec) is ", elapsed_time

      dsize = dble(nsize)
      perf= 2.d0 * dsize * dsize * dsize * dble(iteration) / elapsed_time / 1.d+6 
      write(*,'(1x,a,i5)') "matrix size = ", nsize
      write(*,'(1x,a,f12.5)') "performance (MFLOP/s) = ", perf

!! print a corner in the 1st row of the C matrix.
      do i = 0, 5
         write(*,'(es12.5,1x)')  matrix_c(i*nsize)
      end do

      write(*,*)

      DEALLOCATE(matrix_a,matrix_b,matrix_c)

      end program mkl_dgemm

