!=======================================================!
! The current code is to:                               !
! (1) demonstrate the effect of vec and no-vec.         !
! (2) mesure the perfomance in terms of walltime.       !
! (3) impact of the data size.                          !
!-------------------------------------------------------!
! Xiaoxu Guan at LSU HPC.                               !
! May 16, 2017                                          !
!=======================================================!

      program vectorization_f_v0
      implicit none
      integer i,k,nosize,id,nothread
      real(kind=8), dimension(:), allocatable :: vector_a, &
      vector_b,vector_c,vector_d,vector_e
      integer, parameter :: nsize = 20, kitemax = 10000000
      real(kind=8)       :: time_start,time_end,omp_get_wtime
      integer            :: omp_get_thread_num,omp_get_num_threads

      nosize = nsize

! allocate the vectors.
      ALLOCATE( vector_a(0:nosize-1) )
      ALLOCATE( vector_b(0:nosize-1) )
      ALLOCATE( vector_c(0:nosize-1) )
      ALLOCATE( vector_d(0:nosize-1) )
      ALLOCATE( vector_e(0:nosize-1) )

! initialize the vectors.
      do i = 0, nosize-1
        vector_a(i) = sin(dble(i))
        vector_b(i) = sin(dble(i+i))
        vector_c(i) = cos(dble(i*i))
        vector_d(i) = cos(dble(i+i+i))
        vector_e(i) = sin(dble(i+i+i+i))
      end do 

!$omp parallel
      id = omp_get_thread_num()
      nothread = omp_get_num_threads()
      write(*,'(a,i3,a,i3)')  "This is ID", id, " out of ",nothread
!$omp end parallel
 

! start a timer.
      time_start = omp_get_wtime()

! sum over all vector elements.
      do k = 0, kitemax-1
         do i = 0, nosize-1
            vector_a(i) =  vector_a(i) + vector_b(i) + vector_c(i) + vector_d(i) + vector_e(i)
         end do
      end do

! end of the timer.
      time_end = omp_get_wtime()
      write(*,'(a,f10.5)') "Elapsed time (sec) is ", time_end-time_start

      DEALLOCATE(vector_a,vector_b,vector_c,vector_d,vector_e)

      end program vectorization_f_v0


